﻿using XLua;
using UnityEngine;

/// <summary>
/// XLuaの管理を行うシングルトンマネージャです。
/// スクリプトファイルの読み込み、Lua関数の実行等の操作を提供します。
/// </summary>
public sealed class AbyssLibU_XLua
{
    //シングルトン
    private static AbyssLibU_XLua _instance = new AbyssLibU_XLua();
    public static AbyssLibU_XLua Instance
    {
        get
        {
            return _instance;
        }
    }
    private AbyssLibU_XLua() { }
    //LuaEnvインスタンス
    public LuaEnv LE = new LuaEnv();

    /// <summary>
    /// スクリプトファイルを読み込み実行します。
    /// Resourcesフォルダ⇒外部データの順に読み込みを試みます。
    /// </summary>
    /// <param name="path">スクリプトファイルのパスを指定します。</param>
    public void DoScript(string path)
    {
        try
        {
            LE.DoString("require '" + path + "'");
        }
        catch (LuaException)
        {
            System.IO.StreamReader sr = new System.IO.StreamReader(path, System.Text.Encoding.GetEncoding("UTF-8"));
            LE.DoString(sr.ReadToEnd());
            sr.Close();
        }
    }

    /// <summary>
    /// Lua関数を実行します。
    /// </summary>
    /// <param name="function">関数名を指定します。</param>
    /// <returns>Lua関数による戻り値を返します。</returns>
    public object[] Call(string function)
    {
        return (LE.Global.Get<LuaFunction>(function).Call());
    }

    /// <summary>
    /// Lua関数によるコールバック関数を作成します。
    /// </summary>
    /// <param name="function">関数名を指定します。</param>
    /// <returns>Lua関数によるコールバック関数を返します。</returns>
    public UnityEngine.Events.UnityAction MakeCallback(string function)
    {
        return (() => LE.Global.Get<LuaFunction>(function).Call());
    }
}
